<?php
/*
Plugin Name: Philips hue + WordPress
Plugin URI: http://www.visser.io/tools/philips-hue/
Description: Connect your Philips hue Bridge, Lights, Switches and Sensors to WordPress.
Author: Visser I/O
Author URI: http://www.visser.io
Version: 1.0
Text Domain: philips_hue_lighting
*/

include_once( 'includes/lighting-lights.php' );
include_once( 'includes/lighting-sensors.php' );

function codex_nas_lighting_shortcode( $atts ) {

	$lights = get_transient( 'codex_nas_lighting_lights' );
	if( $lights == false ) {
		codex_nas_lighting_lights_job();
		$lights = get_transient( 'codex_nas_lighting_lights' );
		$timestamp = wp_next_scheduled( 'codex_nas_lighting_lights_job' );
		if( $timestamp == false )
			wp_schedule_event( time(), 'hourly', 'codex_nas_lighting_lights_job' );
	}

	$sensors = get_transient( 'codex_nas_lighting_sensors' );
	if( $sensors == false ) {
		codex_nas_lighting_sensors_job();
		$sensors = get_transient( 'codex_nas_lighting_sensors' );
		$timestamp = wp_next_scheduled( 'codex_nas_lighting_sensors_job' );
		if( $timestamp == false )
			wp_schedule_event( time(), 'hourly', 'codex_nas_lighting_sensors_job' );
	}

	// Check the User Capability
	if( current_user_can( 'manage_options' ) == false ) {
		wp_login_form();
		return;
	}

	ob_start();

	echo '<div id="lighting">';

	// Lights
	if( !empty( $lights ) ) {
		echo '<div id="lighting-lights">';

		echo '<p>';
		echo '<a href="' . add_query_arg( array( 'action' => 'refresh_lights', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Refresh lights', 'philips_hue_lighting' ) . '</a>' . ' | ';
		echo '<a href="' . add_query_arg( array( 'action' => 'lights_on', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Turn lights on', 'philips_hue_lighting' ) . '</a>' . ' | ';
		echo '<a href="' . add_query_arg( array( 'action' => 'lights_off', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Turn lights off', 'philips_hue_lighting' ) . '</a>' . ' | ';
		echo '<a href="' . add_query_arg( array( 'action' => 'flash_lights', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Flash lights', 'philips_hue_lighting' ) . '</a>';
		// echo '<a href="' . add_query_arg( array( 'action' => 'set_location', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Set Location', 'philips_hue_lighting' ) . '</a>';
		echo '</p>';

		echo '<ul id="lights">';
		foreach( $lights as $light ) {
			echo '<li id="light-' . $light['uniqueid'] . ' class="light-id-' . $light['id'] . ( empty( $light['reachable'] ) ? ' light-disabled' : '' ) . '">';
			echo '<strong>' . $light['name'] . '</strong>';
			echo ' (<abbr title="' . $light['manufacturername'] . ' ' . $light['modelid'] . ' - ' . $light['swversion'] . '">' . $light['type'] . '</abbr>)';
			echo '<br />';
			echo '<div id="slider" class="light-slider light-slider-id-' . $light['id'] . '" data-light-id="' . $light['id'] . '" data-brightness="' . $light['brightness'] . '" data-uniqueid="' . $light['uniqueid'] . '"></div>';
			echo '<a href="' . add_query_arg( array( 'action' => ( !empty( $light['on'] ) ? 'light_off' : 'light_on' ), 'light_id' => $light['id'], '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" title="' . ( !empty( $light['on'] ) ? __( 'Turn the light off', 'philips_hue_lighting' ) : __( 'Turn the light on', 'philips_hue_lighting' ) ) . '">' . ( empty( $light['on'] ) ? __( 'Off', 'philips_hue_lighting' ) : __( 'On', 'philips_hue_lighting' ) ) . '</a>';
			echo ' - ';
			echo __( 'Brightness', 'philips_hue_lighting' ) . ': <span class="brightness light-brightness-id-' . $light['id'] . '">' . $light['brightness'] . '</span>';
			if( empty( $light['reachable'] ) ) {
				echo '<p>' . sprintf( __( '%s cannot be reached by the Hue Bridge. Is it turned on at the wall?', 'philips_hue_lighting' ), $light['name'] ) . '</p>';
			}
			echo '<hr />';
			echo '</li>';
		}
		echo '</ul>
<!-- #lights -->';
		echo '<p>' . __( 'Total lights', 'philips_hue_lighting' ) . ': ' . count( $lights ) . '</p>';
		echo '</div>';
		echo '<!-- #lighting-lights -->';
	}
	
	// Sensors
	if( !empty( $sensors ) ) {
		echo '<div id="lighting-sensors">';
		echo '<h3>' . __( 'Sensors', 'philips_hue_lighting' ) . '</h3>';

		echo '<p>';
		echo '<a href="' . add_query_arg( array( 'action' => 'refresh_sensors', '_wpnonce' => wp_create_nonce( 'codex_nas_lighting' ) ) ) . '" class="button">' . __( 'Refresh sensors', 'philips_hue_lighting' ) . '</a>';
		echo '</p>';

		echo '<div class="ajax">';

		echo '<ul id="sensors">';
		$i = 0;
		foreach( $sensors as $sensor_id => $sensor ) {
			echo '<li id="sensor-' . $sensor_id . ' class="sensor-id-' . $sensor_id . ( isset( $sensor['reachable'] ) && empty( $sensor['reachable'] ) ? ' sensor-disabled' : '' ) . '">';
			echo '<strong>' . $sensor['name'] . '</strong>';
			echo ' (<abbr title="' . $sensor['manufacturername'] . ' ' . $sensor['modelid'] . ' - ' . $sensor['swversion'] . ( !empty( $sensor['battery'] ) ? ' - ' . __( 'Battery', 'philips_hue_lighting' ) . ': ' . $sensor['battery'] . '%' : '' ) . '">' . $sensor['type'] . '</abbr>)' . '<br />';
			echo ( !empty( $sensor['on'] ) ? __( 'On', 'philips_hue_lighting' ) : __( 'Off', 'philips_hue_lighting' ) );
			echo ' - ';
			// echo ( $sensor['lastupdated'] <> 'none' ? __( 'Last updated', 'philips_hue_lighting' ) . ': ' . human_time_diff( strtotime( $sensor['lastupdated'] ), current_time( 'timestamp' ) ) . ' ' . __( 'ago', 'philips_hue_lighting' ) : '' );
			echo ( $sensor['lastupdated'] <> 'none' ? __( 'Last updated', 'philips_hue_lighting' ) . ': ' . human_time_diff( strtotime( $sensor['lastupdated'] ), time() ) . ' ' . __( 'ago', 'philips_hue_lighting' ) : '' );
			// echo ( !empty( $sensor['buttonevent'] ) ? ' - ' . __( 'Button event', 'philips_hue_lighting' ) . ': ' . $sensor['buttonevent'] : '' );
			// echo ( isset( $sensor['uniqueid'] ) && !empty( $sensor['uniqueid'] ) ? __( 'Unique ID', 'philips_hue_lighting' ) . ': ' . $sensor['uniqueid'] . '<br />' : '' );
			// echo '<br />';
			// echo 'state: ' . print_r( $sensor->state, true ) . '<br />';
			// echo 'config: ' . print_r( $sensor->config, true ) . '<br />';
			// echo print_r( $sensor, true );
			echo '<hr />';
			echo '</li>';
			$i++;
		}
		echo '</ul>
<!-- #sensors -->';
		echo '<p>' . __( 'Total sensors', 'philips_hue_lighting' ) . ': ' . $i . '</p>';

		echo '</div>';
		echo '</div>';
		echo '<!-- #lighting-sensors -->';
	}

	echo '</div>';
	echo '<!-- #lighting -->';

	$output = ob_get_clean();
	return $output;

}
add_shortcode( 'philips_hue_lighting', 'codex_nas_lighting_shortcode' );

function codex_nas_lighting_init() {

	$ip_address = codex_nas_lighting_get_option( 'bridge_ip', false );
	$username = codex_nas_lighting_get_option( 'bridge_username', false );

	// Check if the Bridge Username and Bridge IP have been provided
	if( $username == false || $ip_address == false )
		return;

	$action = ( isset( $_GET['action'] ) ? sanitize_text_field( $_GET['action'] ) : false );
	if( !empty( $action ) ) {
		switch( $action ) {

			case 'refresh_lights':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					codex_nas_lighting_delete_lights();
					codex_nas_lighting_lights_job();
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'refresh_sensors':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					codex_nas_lighting_delete_sensors();
					codex_nas_lighting_sensors_job();
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'lights_on':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					codex_nas_lighting_turn_lights_on();
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'lights_off':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					codex_nas_lighting_turn_lights_off();
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'light_on':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					$light_id = ( isset( $_GET['light_id'] ) ? sanitize_text_field( $_GET['light_id'] ) : false );
					if( $light_id !== false )
						codex_nas_lighting_turn_light_on( $light_id );
					$url = add_query_arg( array( 'action' => null, 'light_id' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'light_off':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					$light_id = ( isset( $_GET['light_id'] ) ? sanitize_text_field( $_GET['light_id'] ) : false );
					if( $light_id !== false )
						codex_nas_lighting_turn_light_off( $light_id );
					$url = add_query_arg( array( 'action' => null, 'light_id' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'flash_lights':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					codex_nas_lighting_flash_lights();
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

			case 'set_location':
				// We need to verify the nonce.
				if( !empty( $_GET ) && check_admin_referer( 'codex_nas_lighting' ) ) {
					$sensor_id = 1;
					$url = 'http://' . $ip_address . '/api/' . $username . '/sensors/' . $sensor_id . '/config';
					$args = array(
						'method' => 'PUT',
						'body' => '{"long": "53.551085W", "lat": "9.993682N", "sunriseoffset": 30, "sunsetoffset": -30}'
					);
					$response = wp_remote_post( $url, $args );
					error_log( 'set_location: ' . print_r( $response, true ) );
					$url = add_query_arg( array( 'action' => null, '_wpnonce' => null ) );
					wp_redirect( $url );
					exit();
				}
				break;

		}
	}

}
add_action( 'init', 'codex_nas_lighting_init' );

function codex_nas_lighting_flash_lights() {

	$ip_address = codex_nas_lighting_get_option( 'bridge_ip', false );
	$username = codex_nas_lighting_get_option( 'bridge_username', false );

	$lights = get_transient( 'codex_nas_lighting_lights' );
	if( $lights == false ) {
		codex_nas_lighting_lights_job();
		$lights = get_transient( 'codex_nas_lighting_lights' );
	}
	if( !empty( $lights ) ) {
		foreach( $lights as $key => $light ) {
			$url = 'http://' . $ip_address . '/api/' . $username . '/lights/' . $light['id'] . '/state';
			$args = array(
				'method' => 'PUT',
				'body' => '{"alert":"select"}'
			);
			$response = wp_remote_post( $url, $args );
			if( is_wp_error( $response ) ) {
				error_log( 'lights alert: ' . print_r( $response, true ) );
				return false;
			} else {
				$lights[$key]['alert'] = 'select';
			}
		}
		set_transient( 'codex_nas_lighting_lights', $lights, HOUR_IN_SECONDS );
		return true;
	}

}

function codex_nas_lighting_light_brightness( $light_id = 0, $brightness = false ) {

	if( empty( $light_id ) && $brightness == false )
		return;

	$ip_address = codex_nas_lighting_get_option( 'bridge_ip', false );
	$username = codex_nas_lighting_get_option( 'bridge_username', false );

	$url = 'http://' . $ip_address . '/api/' . $username . '/lights/' . $light_id . '/state';

	$args = array(
		'method' => 'PUT',
		'body' => '{"bri":' . $brightness . '}'
	);
	$response = wp_remote_post( $url, $args );
	if( is_wp_error( $response ) ) {
		error_log( 'light brightness: ' . print_r( $response, true ) );
	}

	// Update light brightness
	$lights = get_transient( 'codex_nas_lighting_lights' );
	if( !empty( $lights ) ) {
		foreach( $lights as $key => $light ) {
			if( $light['id'] == $light_id ) {
				$lights[$key]['brightness'] = $brightness;
				set_transient( 'codex_nas_lighting_lights', $lights, HOUR_IN_SECONDS );
				break;
			}
		}
	}
	return true;

}

function codex_nas_lighting_lights_javascript() {

	// Check the User Capability
	if( current_user_can( 'manage_options' ) == false )
		return;

	$post_id = codex_nas_lighting_get_option( 'post_id', false );
	if( $post_id !== false ) {
		// Return the Light Javascript if we are on the Lighting Page
		if( get_the_ID() <> $post_id )
			return;
	}

?>
<script type="text/javascript">
	var $j = jQuery.noConflict();
	$j(function() {

		$j(function() {
			$j( ".light-slider" ).each(function() {
				var value = $j(this).attr('data-brightness');
				$j(this).slider({
					value: value,
					range: "min",
					animate: true,
					max: 255,
					min: 0,
					change: function( event, ui ){
						var light_id = $j(this).attr('data-light-id');
						var uniqueid = $j(this).attr('data-uniqueid');
						var brightness = $j(this).slider("value");
						$j(this).attr('data-brightness', brightness);
						$j( '.light-brightness-id-' + light_id ).text(brightness);
						var data = {
							'action': 'update_light_status',
							'light_id': light_id,
							'uniqueid': uniqueid,
							'brightness': brightness
						};
						var ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>';
						$j.post(ajaxurl, data, function(response) {
							/* alert(response); */
						});
					}
				});
			});
		});

	});

	jQuery(document).ready(function($) {

		var data = {
			'action': 'refresh_lighting_sensors',
		};
		var ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>';

		setInterval(function() {
			$.get(ajaxurl, data, function (result) {
				$( '#lighting-sensors .ajax' ).html( result );
			});
		},10000);

	});
</script>
<?php

}
add_action( 'wp_footer', 'codex_nas_lighting_lights_javascript' );

function codex_nas_lighting_enqueue_styles() {

	wp_enqueue_style( 'codex_nas_lighting', plugins_url( '/css/style.css', basename( dirname( __FILE__ ) ) . '/' . basename( __FILE__ ) ) );

}
add_action( 'wp_enqueue_scripts', 'codex_nas_lighting_enqueue_styles' );

function codex_nas_lighting_lights_ajax() {

	$light_id = ( isset( $_POST['light_id'] ) ? absint( $_POST['light_id'] ) : false );
	$uniqueid = ( isset( $_POST['uniqueid'] ) ? sanitize_text_field( $_POST['uniqueid'] ) : '' );
	$brightness = ( isset( $_POST['brightness'] ) ? absint( $_POST['brightness'] ) : false );

	if( !empty( $light_id ) && $brightness !== false ) {
		codex_nas_lighting_light_brightness( $light_id, $brightness );
	}
	die();

}
add_action( 'wp_ajax_update_light_status', 'codex_nas_lighting_lights_ajax' );
add_action( 'wp_ajax_nopriv_update_light_status', 'codex_nas_lighting_lights_ajax' );

// Register and load the widget
function codex_nas_load_lighting_widgetwidget() {

	register_widget( 'codex_nas_lighting_lights_widget' );

}
add_action( 'widgets_init', 'codex_nas_load_lighting_widgetwidget' );

// Creating the widget
class codex_nas_lighting_lights_widget extends WP_Widget {

	function __construct() {

		parent::__construct(
			// Base ID of your widget
			'codex_nas_lighting_lights_widget',
			// Widget name will appear in UI
			__( 'Lights', 'philips_hue_lighting' ),
			// Widget description
			array( 'description' => __( 'Control the State of your Philip Hue Lights from a Widget.', 'philips_hue_lighting' ) )
		);

	}

	// Creating widget front-end
	// This is where the action happens
	public function widget( $args, $instance ) {

		// Check the User Capability
		if( current_user_can( 'manage_options' ) == false )
			return;

		$post_id = codex_nas_lighting_get_option( 'post_id', false );
		if( $post_id !== false ) {
			// Hide the Lights widget if we are on the Lighting Page
			if( get_the_ID() == $post_id )
				return;
		}

		$title = apply_filters( 'widget_title', $instance['title'] );
		// before and after widget arguments are defined by themes
		echo $args['before_widget'];
		if ( ! empty( $title ) )
		echo $args['before_title'] . $title . $args['after_title'];

		// This is where you run the code and display the output

	$lights = get_transient( 'codex_nas_lighting_lights' );
	if( !empty( $lights ) ) {
		echo '<div class="lights">';
		foreach( $lights as $light ) {
			echo '<p id="light-' . $light['uniqueid'] . ' class="light-id-' . $light['id'] . ( empty( $light['reachable'] ) ? ' light-disabled' : '' ) . '">';
			echo '<strong>' . $light['name'] . '</strong>';
			echo ' (<abbr title="' . $light['manufacturername'] . ' ' . $light['modelid'] . ' - ' . $light['swversion'] . '">' . $light['type'] . '</abbr>)';
			echo '<br />';
			echo '<a href="' . add_query_arg( array( 'action' => ( !empty( $light['on'] ) ? 'light_off' : 'light_on' ), 'light_id' => $light['id'] ) ) . '" title="' . ( !empty( $light['on'] ) ? __( 'Turn the light off', 'codex_nas' ) : __( 'Turn the light on', 'codex_nas' ) ) . '" class="button">' . ( empty( $light['on'] ) ? __( 'Off', 'philips_hue_lighting' ) : __( 'On', 'philips_hue_lighting' ) ) . ' - ' . __( 'Brightness', 'philips_hue_lighting' ) . ': ' . $light['brightness'] . '</a>';
			echo '</p>';
		}
		echo '</div>';
	}

		echo '<hr />';

		echo $args['after_widget'];

	}

	// Widget Backend
	public function form( $instance ) {

		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		}	else {
			$title = __( 'Lights', 'philips_hue_lighting' );
		}
		// Widget admin form
?>
<p>
<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label>
<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
</p>
<?php

	}

	// Updating widget replacing old instances with new
	public function update( $new_instance, $old_instance ) {

		$instance = array();
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		return $instance;

	}

} // Class codex_nas_lighting_lights_widget ends here

if( is_admin() ) {

	function codex_nas_lighting_admin_menu() {

		add_options_page( __( 'Philips hue', 'philips_hue_lighting' ), __( 'Philips hue', 'philips_hue_lighting' ), 'manage_options', 'philips_hue', 'codex_nas_lighting_settings' );

	}
	add_action( 'admin_menu', 'codex_nas_lighting_admin_menu' );

	function codex_nas_lighting_settings_init() {

		register_setting( 'philips_hue_settings', 'philips_hue_settings' );

		add_settings_section( 'codex_nas_lighting_settings', 'General Settings', 'codex_nas_lighting_settings_text', 'philips_hue' );

		add_settings_field( 'codex_nas_lighting_bridge_ip', 'Hue Bridge IP Address', 'codex_nas_lighting_settings_bridge_ip', 'philips_hue', 'codex_nas_lighting_settings' );
		add_settings_field( 'codex_nas_lighting_bridge_username', 'Hue Bridge User', 'codex_nas_lighting_settings_bridge_username', 'philips_hue', 'codex_nas_lighting_settings' );
		add_settings_field( 'codex_nas_lighting_bridge_post_id', 'Post ID', 'codex_nas_lighting_settings_post_id', 'philips_hue', 'codex_nas_lighting_settings' );

	}
	add_action( 'admin_init', 'codex_nas_lighting_settings_init' );

	function codex_nas_lighting_settings() {

?>
<div class="wrap">
	<h2><?php _e( 'Philips hue', 'philips_hue_lighting' ); ?></h2>
	<form method="post" action="options.php">
<?php
	// This prints out all hidden setting fields
	settings_fields( 'philips_hue_settings' );
	do_settings_sections( 'philips_hue' );
	submit_button();
?>
	</form>
	</div>
<?php

	}

	function codex_nas_lighting_settings_text() {

		_e( 'Enter your settings below:', 'philips_hue_lighting' );

	}

	function codex_nas_lighting_settings_bridge_ip() {

		$value = codex_nas_lighting_get_option( 'bridge_ip', '' );

		echo '
<input type="text" name="philips_hue_settings[bridge_ip]" id="bridge_ip" value="' . $value . '" class="code" /> 
<span class="description">'. __( 'Enter the local IP address of your Philips hue Bridge.', 'philips_hue_lighting' ) . '</span>';

	}

	function codex_nas_lighting_settings_bridge_username() {

		$value = codex_nas_lighting_get_option( 'bridge_username', '' );

		echo '
<input type="text" name="philips_hue_settings[bridge_username]" id="bridge_username" value="' . $value . '" class="code" /> 
<span class="description">'. __( 'Enter the user authenticated on your Philips hue Bridge.', 'philips_hue_lighting' ) . ' ' . sprintf( '<a href="%s" target="_blank">' . __( 'Learn more', 'philips_hue_lighting' ) . '</a>', 'http://www.developers.meethue.com/documentation/getting-started' ) . '</span>';

	}

	function codex_nas_lighting_settings_post_id() {

		$value = codex_nas_lighting_get_option( 'post_id', '' );

		echo '
<input type="text" name="philips_hue_settings[post_id]" id="post_id" value="' . $value . '" class="code" /> 
<span class="description">'. __( 'Enter the Post ID of the Page where the Shortcode <code>[philips_hue_lighting]</code> has been added to the Post Content.', 'philips_hue_lighting' ) . '</span>';

	}

}

function codex_nas_lighting_get_option( $name = '', $default = false ) {

	if( empty( $name ) )
		return;

	$options = get_option( 'philips_hue_settings' );
	$value = ( isset( $options[$name] ) ? sanitize_text_field( $options[$name] ) : false );
	if( $value == false && $default !== false )
		$value = $default;
	return $value;

}
?>